/*
  PIM for Quagga
  Copyright (C) 2008  Everton da Silva Marques

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING; if not, write to the
  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
  MA 02110-1301 USA
  
  $Id$
*/

#if 0

#include <zebra.h>

#include "log.h"
#include "thread.h"
#include "memory.h"

#include "pimd.h"
#include "pim_pim.h"
#include "pim_time.h"
#include "pim_iface.h"
#include "pim_sock.h"

void pim_sock_free(struct pim_sock *psock)
{
  assert(!psock->t_sock_read);
  XFREE(MTYPE_PIM_PIM_SOCKET, psock);
}

struct pim_sock *pim_sock_lookup_ifaddr(struct list *pim_sock_list,
					struct in_addr ifaddr)
{
  struct listnode *sock_node;
  struct pim_sock *psock;

  for (ALL_LIST_ELEMENTS_RO(pim_sock_list, sock_node, psock))
    if (ifaddr.s_addr == psock->ifaddr.s_addr)
      return psock;

  return 0;
}

static void sock_close(struct pim_sock *psock)
{
  if (PIM_DEBUG_PIM_TRACE) {
    if (psock->t_sock_read) {
      zlog_debug("Cancelling READ event on PIM socket %s fd=%d on interface %s",
		 inet_ntoa(psock->ifaddr), psock->fd,
		 psock->interface->name);
    }
  }
  THREAD_OFF(psock->t_sock_read);
  assert(!psock->t_sock_read);
  
  if (close(psock->fd)) {
    zlog_err("Failure closing PIM socket %s fd=%d on interface %s: errno=%d: %s",
	     inet_ntoa(psock->ifaddr), psock->fd, psock->interface->name,
	     errno, strerror(errno));
  }
  
  if (PIM_DEBUG_PIM_TRACE) {
    zlog_debug("Deleted PIM socket %s fd=%d on interface %s",
	       inet_ntoa(psock->ifaddr), psock->fd, psock->interface->name);
  }
}

void pim_sock_delete(struct pim_sock *psock)
{
  struct pim_interface *pim_ifp;

  pim_ifp = psock->interface->info;

  sock_close(psock);

  listnode_delete(pim_ifp->pim_socket_list, psock);

  pim_sock_free(psock);
}

static void pim_sock_read_on(struct pim_sock *psock);

static int pim_sock_read(struct thread *t)
{
  struct pim_sock *psock;
  int fd;

  psock = THREAD_ARG(t);
  fd = THREAD_FD(t);

  zlog_debug("%s %s: FIXME WRITEME",
	     __FILE__, __PRETTY_FUNCTION__);

  /*
  pim_sock_read_on(psock);
  */

  return 0;
}

static void pim_sock_read_on(struct pim_sock *psock)
{
  assert(psock);

  if (PIM_DEBUG_PIM_TRACE) {
    zlog_debug("Scheduling READ event on PIM socket fd=%d",
	       psock->fd);
  }
  psock->t_sock_read = 0;
  assert(!psock->t_sock_read);
  THREAD_READ_ON(master, psock->t_sock_read, pim_sock_read, psock, psock->fd);
}

static struct pim_sock *pim_sock_new(int fd,
				       struct in_addr ifaddr,
				       struct interface *ifp)
{
  struct pim_interface *pim_ifp;
  struct pim_sock *psock;

  pim_ifp = ifp->info;

  if (PIM_DEBUG_PIM_TRACE) {
    zlog_debug("Creating PIM socket fd=%d for address %s on interface %s",
	       fd, inet_ntoa(ifaddr), ifp->name);
  }

  psock = XMALLOC(MTYPE_PIM_PIM_SOCKET, sizeof(*psock));
  if (!psock) {
    zlog_warn("%s %s: XMALLOC() failure",
              __FILE__, __PRETTY_FUNCTION__);
    return 0;
  }

  psock->fd            = fd;
  psock->interface     = ifp;
  psock->ifaddr        = ifaddr;
  psock->t_sock_read   = 0;
  psock->sock_creation = pim_time_monotonic_sec();

  pim_sock_read_on(psock);

  return psock;
}

static int pim_sock_open(struct in_addr ifaddr, int ifindex, uint32_t pim_options)
{
  struct in_addr group;
  int fd;

  fd = pim_socket_mcast(IPPROTO_PIM, ifaddr);
  if (fd < 0)
    return -1;

  if (!inet_aton(PIM_ALL_PIM_ROUTERS, &group)) {
    zlog_warn("%s %s: PIM socket fd=%d interface %s: could not solve %s to group address: errno=%d: %s",
	      __FILE__, __PRETTY_FUNCTION__, fd, inet_ntoa(ifaddr),
	      PIM_ALL_PIM_ROUTERS, errno, strerror(errno));
    return -2;
  }

  if (pim_socket_join(fd, group, ifaddr, ifindex)) {
    return -3;
  }

  return fd;
}

struct pim_sock *pim_sock_add(struct list *pim_sock_list,
			      struct in_addr ifaddr,
			      struct interface *ifp)
{
  struct pim_interface *pim_ifp;
  struct pim_sock *psock;
  int fd;

  pim_ifp = ifp->info;

  fd = pim_sock_open(ifaddr, ifp->ifindex, pim_ifp->options);
  if (fd < 0) {
    zlog_warn("Could not open PIM socket for %s on %s",
	      inet_ntoa(ifaddr), ifp->name);
    return 0;
  }

  psock = pim_sock_new(fd, ifaddr, ifp);
  if (!psock) {
    zlog_err("%s %s: pim_sock_new() failure",
	     __FILE__, __PRETTY_FUNCTION__);
    close(fd);
    return 0;
  }

  listnode_add(pim_sock_list, psock);

  return psock;
}

#endif /* #if 0 */
